package pl.edu.icm.unicore.uvos.clc;

import java.io.IOException;
import java.lang.reflect.Constructor;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import jline.ConsoleReader;
import jline.SimpleCompletor;

import pl.edu.icm.unicore.uvos.api.exceptions.AuthenticationException;
import pl.edu.icm.unicore.uvos.api.exceptions.IdentityNotKnownException;
import pl.edu.icm.unicore.uvos.api.exceptions.InternalException;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.client.util.ConfigHelper;
import pl.edu.icm.unicore.uvos.client.util.ConfigLoader;
import pl.edu.icm.unicore.uvos.util.Configuration;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSClientFactory;

public class CLCExecutor {

	private HashMap<String, AbstractAction> actions;

	private String commandLine = null;
	private String parsedLine[] = null;

	private ConfigHelper conf;

	private String configFile = null;

	
	private ConnectionManager connManager;
	
	private boolean interactive = false;
	
	
	private static class ExitAction extends AbstractAction {
		ExitAction(ConnectionManager cm) {
			super(cm, "exit", "Exits application", 0, "quit");
		}

		@Override
		public boolean invoke(String[] args) {
			return false;
		}
	}

	private class HelpAction extends AbstractAction {
		HelpAction(ConnectionManager cm) {
			super(cm, "help",
					"Provides help. Use without arguments to get generic help"
							+ " or with command name as parameter to get help"
							+ " on the specified command usage.");
		}

		@Override
		public boolean invoke(String[] args) {
			if (args.length > 1) {
				AbstractAction a = actions.get(args[1]);
				if (a == null)
					System.out.println("Command " + args[1] + " is not known");
				else
					System.out.println("Help for command: " + args[1] + "\n"
							+ a.getHelp());
			} else {
				System.out.println("This is command line client which operates on "
					+ "UNICORE VO system. Type 'help <CMD>' to get help on "
					+ "every command. Available commands are:\n");
				Iterator<AbstractAction> it = actions.values().iterator();
				List<AbstractAction> list = new ArrayList<AbstractAction>();
				while (it.hasNext()) {
					AbstractAction aa = it.next();
					if (!list.contains(aa))
						list.add(aa);
				}
				Collections.sort(list);
				Collections.reverse(list);
				for (AbstractAction a : list) {
					System.out.print("  " + a.getName());
					for (String alias : a.getAliases())
						System.out.print(" (" + alias + ")");
					System.out.println();
				}
			}
			return true;
		}
	}
	
	private class HelpFullAction extends AbstractAction {
		HelpFullAction(ConnectionManager cm) {
			super(cm, "helpAll", "Provides full help for all commands.", 0);
		}

		@Override
		public boolean invoke(String[] args) {
			System.out.println("This is command line client which operates on "
					+ "UNICORE VO system. Type 'help <CMD>' to get help on "
					+ "every command. Available commands are:");
			Iterator<AbstractAction> it = actions.values().iterator();
			List<AbstractAction> list = new ArrayList<AbstractAction>();
			while (it.hasNext())
				list.add(it.next());
			Collections.sort(list);
			Collections.reverse(list);
			for (AbstractAction a : list) {
				System.out.println("\n------------------------------------\n"
						+ "Command: " + a.getName() + "\n");
				System.out.println(a.getHelp());
			}
			return true;
		}
	}

	
	
	public CLCExecutor(){
		
	}

	public CLCExecutor(ConnectionManager m){
		this.connManager=m;
	}

	public void commandLoop() {
		ConsoleReader in;
		try {
			in = new ConsoleReader();
		} catch (IOException e1) {
			e1.printStackTrace();
			return;
		}
		initReader(in);
		System.out.println("\n"
				+ "This is a command line client which operates on "
				+ "UNICORE VO system.\n  Type 'help' to get started.\n");

		boolean end = false;
		do {
			String cmdLine;
			try {
				cmdLine = in.readLine();
				end = !invokeInteractiveCommand(cmdLine);
			} catch (IOException e) {
				end = true;
				e.printStackTrace();
			} catch (UVOSException e) {
				System.err.println("ERROR OCCURED:");
				e.printStackTrace();
			}

			System.out.println();
		} while (!end);
	}

	public String getCommandLine() {
		return commandLine;
	}

	public String[] getParsedLine() {
		return parsedLine;
	}

	public void init() throws InternalException, IdentityNotKnownException,
			AuthenticationException, MalformedURLException {

		String addr = conf.getProtocol() + "://" + conf.getServerHost() + ":"
				+ conf.getServerPort();

		UVOSClientFactory fact = new UVOSClientFactory(conf, conf
				.getConfiguration());

		connManager = new ConnectionManager(fact.getWSManagementClientHL(addr),
				fact.getWSAuthZClientHL(addr), fact.getWSQueryClientHL(addr),
				fact.getWSQueryHistoryClientHL(addr), fact.getWSAppsClientHL(addr),
				fact.getWSGlueClientHL(addr), null);

		registerAllActions();

	}

	private void initReader(ConsoleReader reader) {
		Iterator<AbstractAction> it = actions.values().iterator();
		List<AbstractAction> list = new ArrayList<AbstractAction>();
		SimpleCompletor completor = new SimpleCompletor(new String[] {});
		while (it.hasNext()) {
			AbstractAction aa = it.next();
			completor.addCandidateString(aa.cmd);
			list.add(aa);
		}
		reader.addCompletor(completor);
		reader.setDefaultPrompt("> ");
		reader.setUseHistory(true);
		reader.setUsePagination(true);
	}

	public boolean isInteractive() {
		return interactive;
	}

	public boolean invokeInteractiveCommand(String cmdLine) throws UVOSException {
		String[] tokens = splitCmd(cmdLine);
		return parseLine(tokens);
	}
	
	public boolean parseLine(String[] tokens) throws UVOSException {
		if (tokens.length == 0 || tokens[0].equals(""))
			return true;

		String cmd = tokens[0];
		AbstractAction aa = actions.get(cmd);
		if (aa == null) {
			System.out.println("Command '" + cmd + "' not found."
					+ " Type 'help' to get started.");
			return true;
		}
		if (tokens.length - 1 < aa.getMinArgsNumber()) {
			System.out.println("There must be at least "
					+ aa.getMinArgsNumber() + " arguments to the '"
					+ aa.getName() + "' command.");
			return true;
		}
		return aa.invoke(tokens);
	}

	private void printCLHelp() {
		System.out.println("This is a command line client which operates on a "
			+ "UNICORE VO system.\nIt can be used in both interactive and "
			+ "batch mode.\nThe basic syntax is:\n"
			+ "   uvoscmd.??? [-c <configFile>] <-i|-b COMMAND ...>\n"
			+ "Options are:\n"
			+ "     -i   Enter an intearactive mode.\n"
			+ "     -b   Use batch mode. COMMAND is one of the commands used in "
			+ "interactive mode.\n"
			+ "          Use 'help' command to see the list or 'help <CMD>' "
			+ "to get help on every command.\n\n"
			+ "     -c   Configuration file location.\n"
			+ "Without options this help message is printed. "
			+ "Options order must be preserved.\n\n"
			+ "By default the configuration file is read from the "
			+ ConfigLoader.DEF_CONFIG + " file.\n"
			+ "This can be changed by setting the "
			+ ConfigLoader.CONFIG_LOC_ENV
			+ " environment variable or by -c option.");
	}

	public boolean processCommandLineAgrs(String[] args) {
		if (args.length == 0) {
			printCLHelp();
			return false;
		}
		int nextOpt = 0;
		if (args[nextOpt].equals("-c")) {
			if (args.length < 3) {
				printCLHelp();
				return false;
			}
			nextOpt += 2;
			configFile = args[1];
		}
		if (args[nextOpt].equals("-i")) {
			interactive = true;
			return true;
		} else if (args[nextOpt].equals("-b") && args.length > nextOpt) {
			parsedLine = new String[args.length - nextOpt - 1];
			for (int i = nextOpt + 1,j=0; i < args.length; i++,j++)
				parsedLine[j] = args[i];
			return true;
		}
		printCLHelp();
		return false;
	}

	public void readConfig() {
		conf = null;
		try {
			Configuration c = ConfigLoader.readConfig(configFile);
			conf = new ConfigHelper(c);
		} catch (Exception e1) {
			System.out.println("Can't read configuration: " + e1);
			System.exit(1);
		}
	}

	// --------- CORE ACTIONS --------------

	private void registerAction(Class<? extends AbstractAction> aa) {
		AbstractAction aaObj;
		try {
			Constructor<? extends AbstractAction> constructor = aa
					.getDeclaredConstructor(ConnectionManager.class);
			aaObj = constructor.newInstance(connManager);
		} catch (Exception e) {
			System.err.println("Buggy action: can't register action " + aa);
			e.printStackTrace();
			return;
		}
		actions.put(aaObj.getName(), aaObj);
		String[] aliases = aaObj.getAliases();
		for (String alias : aliases)
			actions.put(alias, aaObj);
	}

	public void registerAllActions() {
		actions = new HashMap<String, AbstractAction>();

		AbstractAction aaObj;
		aaObj = new HelpAction(connManager);
		actions.put(aaObj.getName(), aaObj);
		aaObj = new HelpFullAction(connManager);
		actions.put(aaObj.getName(), aaObj);

		registerAction(ExitAction.class);
		registerAction(AddGroupAction.class);
		registerAction(RemoveGroupAction.class);
		registerAction(AddIdentityAction.class);
		registerAction(AddIdentityWithEquivAction.class);
		registerAction(RemoveIdentityAction.class);
		registerAction(AddToGroupAction.class);
		registerAction(RemoveFromGroupAction.class);
		registerAction(SetLabelAction.class);
		registerAction(CopyGroupAction.class);
		registerAction(SetAttributeAction.class);
		registerAction(RemoveAttributeAction.class);
		registerAction(DisableAttributeAction.class);
		registerAction(EnableAttributeAction.class);
		registerAction(ListDisabledAttributesAction.class);
		registerAction(ListATypesAction.class);
		registerAction(ListITypesAction.class);
		registerAction(PurgeAction.class);
		registerAction(IsMemeberAction.class);
		registerAction(GetGroupContentAction.class);
		registerAction(GetAllAttributesAction.class);
		registerAction(GetAttributeAction.class);
		registerAction(GetAllEquivalentsAction.class);
		registerAction(GetAllGroupsAction.class);
		registerAction(AreEquivalentAction.class);
		registerAction(GetAllIdentitiesAction.class);
		registerAction(RemoveAuthZAction.class);
		registerAction(SetAuthZAction.class);
		registerAction(GetAuthZAction.class);
		registerAction(GetPermissionsAction.class);
		registerAction(GetMyIdentitiesAction.class);
		registerAction(ChangePasswdAction.class);
		registerAction(DeleteATypeAction.class);
		registerAction(UpdateATypeAction.class);
		registerAction(SetTimeAction.class);
		registerAction(GetEventsAction.class);
		registerAction(SetApplicationFormAction.class);
		registerAction(GetApplicationFormsAction.class);
		registerAction(RemoveApplicationFormAction.class);
		registerAction(ListApplicationsAction.class);
		registerAction(ProcessApplicationAction.class);
		registerAction(UpdateCSRApplicationAction.class);
		registerAction(SubmitApplicationAction.class);
		registerAction(SetIdentityStatusAction.class);
		registerAction(ListNotificationsAction.class);
		registerAction(RemoveNotificationAction.class);
		registerAction(AddNotificationAction.class);
		registerAction(GetApplicationAction.class);
		registerAction(GetGlueInfoAction.class);
	}

	private String[] splitCmd(String line) {
		ArrayList<String> ret = new ArrayList<String>();
		char[] c = line.toCharArray();
		char context = ' ';
		StringBuffer curW = new StringBuffer();

		for (int i = 0; i < c.length; i++) {
			if (context != '-') {
				switch (context) {
				case ' ':
					if (c[i] == ' ')
						break;
					if (c[i] == '\'' || c[i] == '"')
						context = c[i];
					else {
						context = '-';
						curW.append(c[i]);
					}
					break;
				case '\'':
				case '"':
					if (c[i] == context) {
						context = '-';
						ret.add(curW.toString());
						curW.setLength(0);
					} else
						curW.append(c[i]);
					break;
				}
			} else {
				switch (c[i]) {
				case ' ':
					if (curW.length() > 0) {
						ret.add(curW.toString());
						curW.setLength(0);
					}
					context = c[i];
					break;
				default:
					curW.append(c[i]);
				}
			}
		}
		if (curW.length() > 0)
			ret.add(curW.toString());
		return ret.toArray(new String[ret.size()]);
	}

}
