/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Mar 14, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.api;

import java.util.List;

import pl.edu.icm.unicore.uvos.api.AttributePermission;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.PermissionDesignator;
import pl.edu.icm.unicore.uvos.api.Permissions;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;

/**
 * This interface encapsulates all externally available operations 
 * regarding VO access authorization that can be used with the UVOS software.
 * 
 * It is for external usage (e.g. via web services), not internal as specified by 
 * {@link UVOSAuthZInterface}. The differences are minor (lack of security context).
 *
 * @author K. Benedyczak
 */
public interface UVOSAuthZExtInterface
{
	/**
	 * Modifies permissions of the group. 
	 * <p>
	 * If attribute name in designator 
	 * has already assigned permissions in this group then those are modified. In this case
	 * if the already existing attribute designator has other values then those in the
	 * new designator, the values of designator attribute are updated (but not added).
	 * <p>
	 * If there is no such attribute designator for the group the permissions are added. 
	 * <p>
	 * If the permissions attribute is null then the entry for designator is removed.
	 * <p>
	 * Requires write perm for the group.
	 * @param group Pass null to modify global permissions
	 * @param designator owner of what attributes will be granted permissions
	 * @param permissions set of permissions to be granted, or null to remove permissions.
	 * @throws UVOSException
	 * @param ctx
	 */
	public void modifyPermissions(Group group, PermissionDesignator designator,
			Permissions permissions)
		throws UVOSException;

	/**
	 * Retrieves set of permissions for the given identity in the group. 
	 * <p>
	 * Requires read perm for the group. Can be self access.
	 * @param group
	 * @param whose
	 * @param ctx
	 * @throws UVOSException
	 * @return
	 */
	public Permissions checkPermissions(Group group, Identity whose)
		throws UVOSException;
	
	/**
	 * Retrieves the identities of the caller.
	 * <p>
	 * Requires no perm.
	 * @return
	 * @throws UVOSException
	 */
	public List<Identity> getMyIdentities() throws UVOSException;
	
	/**
	 * Retrieves specification of authZ settings of the given group.
	 * <p>
	 * Requires write perm for the group.
	 * @param group
	 * @param effective if true then permissions which are effective are returned 
	 * (so those can be permissions inherited from parent group). Otherwise only exact group
	 * configuration is returned.  
	 * @throws UVOSException
	 * @param ctx
	 * @return
	 */
	public List<AttributePermission> getGroupAuthZ(Group group, boolean effective)
		throws UVOSException;
	
	/**
	 * Changes authentication token of the given identity.
	 * <p>
	 * Requires either global write perm or self access (i.e. is self access, 
	 * and also user can modify it's own passwd).
	 * @param id
	 * @param newToken
	 * @param ctx
	 * @throws UVOSException
	 */
	public void modifyAuthenticationData(Identity id, Object newToken)
		throws UVOSException;
}
