/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Aug 17, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.api;

import java.net.MalformedURLException;
import java.net.URL;
import java.security.cert.X509Certificate;
import java.util.List;
import java.util.Properties;

import javax.jws.WebService;

import org.codehaus.xfire.aegis.AegisBindingProvider;
import org.codehaus.xfire.annotations.AnnotationServiceFactory;
import org.codehaus.xfire.xmlbeans.XmlBeansTypeRegistry;
import org.codehaus.xfire.xmlbeans.XmlBeansWSDLBuilderFactory;

import eu.unicore.security.etd.TrustDelegation;
import eu.unicore.security.util.client.HttpUtils;
import eu.unicore.security.util.client.IAuthenticationConfiguration;
import eu.unicore.security.xfireutil.client.ClientTrustDelegationUtil;
import eu.unicore.security.xfireutil.client.XFireClientFactory;

import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.CISInfoProvider;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.SAMLAuthnInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.SAMLNameIdMappingInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.SAMLQueryInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.WSApplicationsInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.WSAuthZInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.WSManagementInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.WSQueryHistoryInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.WSQueryInterface;

/**
 * The client library proxies factory. It allows for creating:
 * <ul>
 * <li> plain proxies for which users need to handle XmlBeans arguments 
 * creation manually 
 * <li> high-level client objects that acts on convenient Java objects.
 * </ul>
 * <p>
 * The factory can be configured with security settings. The settings are used to 
 * preconfigure all proxies that are produced by this factory. 
 * 
 * @author K. Benedyczak
 */
public class UVOSClientFactory extends XFireClientFactory
{
	private final String nameIdMapServiceName, samlQueryServiceName, 
		samlAuthServiceName, manServiceName, authZServiceName, 
		queryServiceName, queryHistoryServiceName, appsServiceName, CISServiceName;
	
	private List<TrustDelegation> tdChain;
	private X509Certificate user;
	private String callerDN;
	private static final int DEFAULT_HTTP_TIMEOUT = 10000;
	
	
	public UVOSClientFactory(IAuthenticationConfiguration secCfg, Properties settings)
	{
		super(getServiceFactory(), secCfg, settings);
		if (!settings.containsKey(HttpUtils.SO_TIMEOUT))
			settings.setProperty(HttpUtils.SO_TIMEOUT, DEFAULT_HTTP_TIMEOUT+"");
		if (!settings.containsKey(HttpUtils.CONNECT_TIMEOUT))
			settings.setProperty(HttpUtils.CONNECT_TIMEOUT, DEFAULT_HTTP_TIMEOUT+"");

		WebService nameMapWS = 
			SAMLNameIdMappingInterface.class.getAnnotation(WebService.class);
		nameIdMapServiceName = nameMapWS.serviceName();
		WebService samlQueryWS = 
			SAMLQueryInterface.class.getAnnotation(WebService.class);
		samlQueryServiceName = samlQueryWS.serviceName();
		WebService samlAuthWS = 
			SAMLAuthnInterface.class.getAnnotation(WebService.class);
		samlAuthServiceName = samlAuthWS.serviceName();
		WebService manWS = 
			WSManagementInterface.class.getAnnotation(WebService.class);
		manServiceName = manWS.serviceName();
		WebService queryWS = 
			WSQueryInterface.class.getAnnotation(WebService.class);
		queryServiceName = queryWS.serviceName();
		WebService queryHistWS = 
			WSQueryHistoryInterface.class.getAnnotation(WebService.class);
		queryHistoryServiceName = queryHistWS.serviceName();
		WebService authZWS = 
			WSAuthZInterface.class.getAnnotation(WebService.class);
		authZServiceName = authZWS.serviceName();
		WebService appsWS = 
			WSApplicationsInterface.class.getAnnotation(WebService.class);
		appsServiceName = appsWS.serviceName();
		WebService glueWS = 
				CISInfoProvider.class.getAnnotation(WebService.class);
		CISServiceName = glueWS.serviceName();	
	}

	private static AnnotationServiceFactory getServiceFactory()
	{
		AnnotationServiceFactory asf = new AnnotationServiceFactory();
		AegisBindingProvider provider = new AegisBindingProvider(
				new XmlBeansTypeRegistry());
		asf.setBindingProvider(provider);
		asf.setWsdlBuilderFactory(new XmlBeansWSDLBuilderFactory());
		return asf;
	}
	
	/**
	 * Sets trust delegation settings for all the
	 * proxies created after this call.
	 * @param tdChain TD chain to be used. User assertion is derived form it - 
	 * it will be TD chain initial issuer (custodian).  
	 */
	public void setTD(List<TrustDelegation> tdChain)
	{
		this.tdChain = tdChain;
		this.callerDN = null;
		this.user = null;
	}
	
	/**
	 * Sets trust delegation settings for all the
	 * proxies created after this call.
	 * @param tdChain TD chain to be used. 
	 * @param user Certificate of user to create User assertion
	 * @param callerDN DN of consignor (caller of this method) to 
	 * be put in User assertion.
	 */
	public void setTD(List<TrustDelegation> tdChain, X509Certificate user,
			String callerDN)
	{
		this.tdChain = tdChain;
		this.callerDN = callerDN;
		this.user = user;
	}
	
	/**
	 * Removes trust delegation insertions for all the proxies created 
	 * after this call.
	 */
	public void removeTD()
	{
		tdChain = null;
		callerDN = null;
		user = null;
	}
	
	/**
	 * Setups security for the proxy. Also connection details are set - HTTP timeouts. 
	 * @param proxy
	 */
	private void setupProxy(Object proxy)
	{
		if (tdChain != null)
		{
			if (callerDN != null && user != null)
				ClientTrustDelegationUtil.addTrustDelegation(
						proxy, tdChain, user, callerDN);
			else
				ClientTrustDelegationUtil.addTrustDelegation(
						proxy, tdChain);
		}
	}

	private String manglePath(String addr, String defName) throws MalformedURLException
	{
		String path = new URL(addr).getPath();
		return (path.equals("")) ? 
				addr + "/" + defName : addr; 
	}
	
	public SAMLQueryInterface getSAMLQueryClient(String addr) 
		throws MalformedURLException
	{
		SAMLQueryInterface samlProxy = createPlainWSProxy(SAMLQueryInterface.class, 
					manglePath(addr, samlQueryServiceName));
		setupProxy(samlProxy);
		return samlProxy;
	}

	public SAMLAuthnInterface getSAMLAuthnClient(String addr) 
		throws MalformedURLException
	{
		SAMLAuthnInterface samlProxy = createPlainWSProxy(SAMLAuthnInterface.class, 
				manglePath(addr, samlAuthServiceName));
		setupProxy(samlProxy);
		return samlProxy;
	}

	public SAMLNameIdMappingInterface getSAMLIdMappingClient(String addr) 
		throws MalformedURLException
	{
		SAMLNameIdMappingInterface samlProxy = createPlainWSProxy(SAMLNameIdMappingInterface.class, 
				manglePath(addr, nameIdMapServiceName));
		setupProxy(samlProxy);
		return samlProxy;
	}
	
	public WSManagementInterface getWSManagementClient(String addr) 
		throws MalformedURLException
	{
		WSManagementInterface manProxy = createPlainWSProxy(WSManagementInterface.class, 
				manglePath(addr, manServiceName));
		setupProxy(manProxy);
		return manProxy;
	}

	public WSManagementClient getWSManagementClientHL(String addr) 
		throws MalformedURLException
	{
		return new WSManagementClient(getWSManagementClient(addr));
	}	
	
	public WSAuthZInterface getWSAuthZClient(String addr) 
		throws MalformedURLException
	{
		WSAuthZInterface manProxy = createPlainWSProxy(WSAuthZInterface.class, 
				manglePath(addr, authZServiceName));
		setupProxy(manProxy);
		return manProxy;
	}
	
	public WSAuthZClient getWSAuthZClientHL(String addr)
		throws MalformedURLException
	{
		return new WSAuthZClient(getWSAuthZClient(addr));
	}

	public WSApplicationsInterface getWSAppsClient(String addr) 
		throws MalformedURLException
	{
		WSApplicationsInterface proxy = createPlainWSProxy(WSApplicationsInterface.class, 
				manglePath(addr, appsServiceName));
		setupProxy(proxy);
		return proxy;
	}

	public WSApplicationClient getWSAppsClientHL(String addr)
		throws MalformedURLException
	{
		return new WSApplicationClient(getWSAppsClient(addr));
	}

	public WSQueryInterface getWSQueryClient(String addr) 
		throws MalformedURLException
	{
		WSQueryInterface queryProxy = createPlainWSProxy(WSQueryInterface.class, 
				manglePath(addr, queryServiceName));
		setupProxy(queryProxy);
		return queryProxy;
	}
	
	public WSQueryClient getWSQueryClientHL(String addr) 
		throws MalformedURLException
	{
		return new WSQueryClient(getWSQueryClient(addr));
	}	

	public WSQueryHistoryInterface getWSQueryHistoryClient(String addr) 
		throws MalformedURLException
	{
		WSQueryHistoryInterface queryProxy = createPlainWSProxy(WSQueryHistoryInterface.class, 
				manglePath(addr, queryHistoryServiceName));
		setupProxy(queryProxy);
		return queryProxy;
	}

	public WSQueryHistoryClient getWSQueryHistoryClientHL(String addr) 
		throws MalformedURLException
	{
		return new WSQueryHistoryClient(getWSQueryHistoryClient(addr));
	}	

	public CISInfoProvider getWSGlueClient(String addr) 
			throws MalformedURLException
	{
		CISInfoProvider queryProxy = createPlainWSProxy(CISInfoProvider.class, 
				manglePath(addr, CISServiceName));
		setupProxy(queryProxy);
		return queryProxy;
	}

	public WSGLUEClient getWSGlueClientHL(String addr) 
			throws MalformedURLException
	{
		return new WSGLUEClient(getWSGlueClient(addr));
	}	
}



