/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Mar 14, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.api;

import java.util.List;

import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Element;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;

/**
 * This interface encapsulates all externally available operations 
 * regarding quering VO contents that can be used with the UVOS software.
 * 
 * It is for external usage (e.g. via web services), not internal as specified by 
 * {@link UVOSQueryInterface}. The differences are minor (lack of security context).
 * @author K. Benedyczak
 */
public interface UVOSQueryExtInterface
{
	/**
	 * Checks if given identity is a memeber of the given group.
	 * <p>
	 * Requires read perm in the group. Can be self access.
	 * @param who
	 * @param group 
	 * @param effective if true method will check also membership which is implied by
	 * membership in one of gorup subgroup otherwise only membership in the group exactely
	 * is checked. 
	 * @throws UVOSException
	 * @return
	 */
	public boolean isMember(Identity who, Group group, boolean effective) 
		throws UVOSException;
	
	/**
	 * Retrieves attributes of the given element (i.e. identity, group or 
	 * identity in a group scope). If URN is null or "" then all attributes are returned.
	 * <p>
	 * See description of parameters to get possible versions of invocation.
	 * <p>
	 * Requires global read perm, can be self access. Without global read
	 * perm parameters valid in groups where caller has no read perm are filtered out.
	 * @param owner
	 * @param attribute the name of interesting attributes, can be null. 
	 * @param effective if true method will get effective attributes e.g. which are implied by
	 * membership in one of group subgroups, otherwise only attributes exactely 
	 * matching are returned.
	 * @param includeScoped important only when owner is an identity 
	 * (not a group or id@group). Then if it is false only global attributes are returned. 
	 * Otherwise attributes global and all group-scoped attributes (for groups where owner 
	 * is a member) are returned.
	 * @param includeImplied Used only when includeScoped is true. Then it governs the set 
	 * of groups which are used to check group-scoped attributes for the owner. If false
	 * only groups where owner is a real memeber are checked. If true then also groups
	 * where membership is implied (by being in group's subgroup) are evaluated. 
	 * @return
	 * @throws UVOSException
	 */
	public List<Attribute> getAttributes(Element owner, String attribute, boolean effective,
			boolean includeScoped, boolean includeImplied) 
		throws UVOSException;
	
	/**
	 * Get all groups the given identity is a memeber of. Doesn't return parent 
	 * groups (where membership is implied by membership in subgroup).
	 * <p>
	 * Requires global read perm, can be self access.
	 * @param who
	 * @return
	 * @throws UVOSException
	 */
	public List<Group> getAllGroups(Identity who, boolean implied) 
		throws UVOSException;
	
	/**
	 * Checks if two identities represent the same entity.
	 * <p>
	 * Requires global identityCtl perm. Can be self access.
	 * @param i1
	 * @param i2
	 * @return
	 * @throws UVOSException
	 */
	public boolean areEquivalent(Identity i1, Identity i2) 
		throws UVOSException;
	
	/**
	 * Retrieves all equivalent identities of the given one.
	 * <p>
	 * Requires global identityCtl perm. Can be self access.
	 * @param who
	 * @return
	 * @throws UVOSException
	 */
	public List<Identity> getAllEquivalents(Identity who) 
		throws UVOSException;

	/**
	 * Retrieves content of the group. Elements are either other groups or identities.
	 * <p>
	 * Requires global or group read perm.
	 * @param group
	 * @return
	 * @throws UVOSException
	 */
	public List<Element> getGroupContent(Group group)
		throws UVOSException;
	
	/**
	 * Retrievies all identities stored in the database. Also those not assigned
	 * to any group. 
	 * <p>
	 * Requires global read perm.
	 * @return
	 * @throws UVOSException
	 */
	public List<Identity> getAllIdentities()
		throws UVOSException;
}
