/*
 * Copyright (c) 2011 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 03-09-2011
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package pl.edu.icm.unicore.uvos.wsclient.api;

import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.CISInfoProvider;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UVOSFault;
import xmlbeans.pl.edu.icm.uvos.cisprovider.GetServiceInfoRequestDocument;
import xmlbeans.pl.edu.icm.uvos.cisprovider.GetServiceInfoResponseDocument;
import xmlbeans.pl.edu.icm.uvos.glue2.EndpointT;
import xmlbeans.pl.edu.icm.uvos.glue2.ServiceT;

/**
 * Client for querying the GLUE endpoint.
 * 
 * @author K. Benedyczak
 */
public class WSGLUEClient implements UVOSGLUEExtInterface
{
	private CISInfoProvider proxy;
	
	/**
	 * Constructor using wrapping a WS proxy.
	 * @param proxy
	 */
	public WSGLUEClient(CISInfoProvider proxy)
	{
		this.proxy = proxy;
	}
	
	
	private EndpointT getFirstEndpoint() throws UVOSException
	{
		ServiceT service = getServicesInformation()[0];
		EndpointT[] endpoints = service.getEndpointArray();
		if (endpoints == null || endpoints.length == 0)
			throw new UVOSException("Got empty information about service's endpoints, protocol failure.");
		return endpoints[0];
	}
	
	/**
	 * @return a string with a server's version. 
	 * @throws UVOSException
	 */
	public String getServerVersion() throws UVOSException
	{
		return getFirstEndpoint().getImplementationVersion();
	}
	
	/**
	 * @return a multiline string which contains information on server's distribution,
	 * or empty string if distribution information is unknown.
	 * @throws UVOSException
	 */
	public String getServerDistribution() throws UVOSException
	{
		EndpointT endpoint = getFirstEndpoint();
		String[] others = endpoint.getOtherInfoArray();
		if (others == null || others.length == 0)
			return "";
		StringBuilder sb = new StringBuilder();
		for (String other: others)
		{
			if (other.startsWith("MiddlewareName="))
				sb.append(other.replace("=", ": ")).append("\n");
			if (other.startsWith("MiddlewareVersion="))
				sb.append(other.replace("=", ": ")).append("\n");
		}
		return sb.toString().trim();
	}
	
	/**
	 * @return a multiline string with basic information about the server including its
	 * name, version and distribution.
	 * @throws UVOSException
	 */
	public String getServerDescription() throws UVOSException
	{
		ServiceT service = getServicesInformation()[0];
		if (service.getEndpointArray() == null || service.getEndpointArray().length == 0)
			throw new UVOSException("Got empty information about service's endpoints, protocol failure.");
		EndpointT endpoint = service.getEndpointArray(0);
		StringBuilder sb = new StringBuilder();
		sb.append("Name: ");
		sb.append(service.getName());
		sb.append("\nVersion: ");
		sb.append(endpoint.getImplementationVersion());
		String distro = getServerDistribution();
		if (distro.length() > 0)
			sb.append("\n").append(distro);
		return sb.toString();
	}
	
	/**
	 * @return an array with GLUE 2 description of the services as retrieved from the server. 
	 * @throws UVOSException
	 */
	public ServiceT[] getServicesInformation() throws UVOSException
	{
		GetServiceInfoResponseDocument serviceInfoDoc = getServiceInfo();
		ServiceT[] services = serviceInfoDoc.getGetServiceInfoResponse().getServiceArray();
		if (services == null || services.length == 0)
			throw new UVOSException("Got empty information about services, protocol failure.");
		return services;
	}
	
	/**
	 * @return a raw GLUE 2 document as retrieved from the server. 
	 * @throws UVOSException
	 */
	public GetServiceInfoResponseDocument getServiceInfo() throws UVOSException
	{
		GetServiceInfoRequestDocument reqDoc = 
				GetServiceInfoRequestDocument.Factory.newInstance();
		reqDoc.addNewGetServiceInfoRequest();
		try
		{
			return proxy.getServiceInfo(reqDoc);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}
	
}
