/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 24, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.samlapi;

import java.io.FileInputStream;
import java.net.MalformedURLException;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;

import eu.unicore.samly2.assertion.Assertion;
import eu.unicore.samly2.proto.AbstractRequest;
import eu.unicore.samly2.proto.AbstractStatusResponse;
import eu.unicore.security.SecurityException;
import eu.unicore.security.dsig.DSigException;
import eu.unicore.security.util.client.IAuthenticationConfiguration;

/**
 * This class provides a base for Client implementations.
 * It merely provides code to handle signatures.
 *
 * @author K. Benedyczak
 */
public abstract class AbstractSAMLBase
{
	protected boolean requireSignedResp, requireSignedAssertion, doSignRequest;
	protected PublicKey publicKey;

	private transient PrivateKey privKey;
	protected X509Certificate[] certificateC;

	protected AbstractSAMLBase(IAuthenticationConfiguration secCfg) 
		throws SecuritySetupException, MalformedURLException
	{
		initSecData(secCfg);
		requireSignedAssertion = false;
		requireSignedResp = false;
		doSignRequest = false;
	}
	
	private void initSecData(IAuthenticationConfiguration secProv) 
		throws SecuritySetupException
	{
		if (secProv.getKeystore() == null)
			throw new SecurityException("SAML client configuration: keystore should not be null");
		if (secProv.getKeystoreType() == null)
			throw new SecurityException("SAML client configuration: keystore type should not be null");
		if (secProv.getKeystorePassword() == null)
			throw new SecurityException("SAML client configuration: keystore password should not be null");
		if (secProv.getKeystoreKeyPassword() == null)
			throw new SecurityException("SAML client configuration: keystore key password should not be null");
		if (secProv.getKeystoreAlias() == null)
			throw new SecurityException("SAML client configuration: keystore key alias should not be null");
		try
		{
			KeyStore ks = KeyStore.getInstance(secProv.getKeystoreType());
			ks.load(new FileInputStream(secProv.getKeystore()), 
				secProv.getKeystorePassword().toCharArray());
			privKey = (PrivateKey) ks.getKey(secProv.getKeystoreAlias(), 
				secProv.getKeystoreKeyPassword().toCharArray());
			Certificate[] certs = ks.getCertificateChain(
					secProv.getKeystoreAlias());
			certificateC = new X509Certificate[certs.length];
			for (int i=0; i< certs.length; i++)
				certificateC[i] = (X509Certificate) certs[i];
		} catch (Exception e)
		{
			throw new SecuritySetupException(e);
		}
	}
	
	/**
	 * Set signing policy, i.e. what is required and checked in received SAML assertions. 
	 * @param requireSignedResp We require that the response must be signed. Usually this is not needed.
	 * This argument also controls whether queries are signed.
	 * @param requireSignedAssertion We require that the assertion in response should be signed.
	 * @param publicKey Public key which should be used for signature verification. May be null -
	 * then signature will be checked with the key of the issuer certificate if it is 
	 * found in the response. 
	 */
	public void setDSigPolicy(boolean requireSignedResp, boolean requireSignedAssertion,
			PublicKey publicKey)
	{
		this.requireSignedAssertion = requireSignedAssertion;
		this.requireSignedResp = requireSignedResp;
		this.doSignRequest = requireSignedResp;
		this.publicKey = publicKey;
	}

	/*-********************************************************
	 * utility methods 
	 *-********************************************************/
	
	protected void signIfNeeded(AbstractRequest req) throws InvalidSignatureException
	{
		if (doSignRequest)
		{
			try
			{
				req.sign(privKey, certificateC);
			} catch (DSigException e)
			{
				throw new InvalidSignatureException("Can't sign request: " 
						+ e);
			}
		}	
	}
	
	protected void checkResponseSignature(AbstractStatusResponse resp) 
		throws InvalidSignatureException
	{
		if (!requireSignedResp)
			return;
		if (publicKey != null)
		{
			try
			{
				if (!resp.isCorrectlySigned(publicKey))
					throw new InvalidSignatureException(
					"Response is incorrectly signed");
			} catch (DSigException e)
			{
				throw new InvalidSignatureException(
					"Problem when checking " +
					"response signature with given pub key: " + e);
			}
		} else
		{
			X509Certificate[] issuersCC = resp.getIssuerFromSignature();
			if (issuersCC == null || issuersCC.length == 0)
				throw new InvalidSignatureException(
						"Response doesn't contain " +
						"issuer's certificate and public key" +
				" for verification wasn't specified");
			try
			{
				if (!resp.isCorrectlySigned(issuersCC[0].getPublicKey()))
					throw new InvalidSignatureException(
					"Response is incorrectly signed");
			} catch (DSigException e)
			{
				throw new InvalidSignatureException(
					"Problem when checking " +
					"response signature with extracted pub key: " + e);
			}
		}
	}

	protected void checkAssertionSignature(AbstractStatusResponse resp, 
			Assertion assertion) throws InvalidSignatureException
	{
		if (!requireSignedAssertion)
			return;
		if (publicKey != null)
		{
			try
			{
				if (!assertion.isCorrectlySigned(publicKey))
					throw new InvalidSignatureException(
					"Assertion is incorrectly signed");
			} catch (DSigException e)
			{
				throw new InvalidSignatureException("Problem when checking " +
						"response assertion signature with given " +
						"pub key: " + e);
			}
		} else
		{
			X509Certificate[] issuersCC = assertion.
				getIssuerFromSignature();
			if (issuersCC == null || issuersCC.length == 0)
			{
				issuersCC = resp.getIssuerFromSignature();
				if (issuersCC == null || issuersCC.length == 0)
					throw new InvalidSignatureException(
						"Assertion doesn't contain " +
						"issuer's certificate and public key" +
						" for verification wasn't specified");
			}
			try
			{
				if (!assertion.isCorrectlySigned(issuersCC[0].getPublicKey()))
					throw new InvalidSignatureException(
					"Response is incorrectly signed");
			} catch (DSigException e)
			{
				throw new InvalidSignatureException("Problem when checking " +
						"response assertion signature with extracted " +
						"pub key: " + e);
			}
		}
	}
	
}
