/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 24, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.samlapi;

import java.net.MalformedURLException;
import java.net.URI;
import java.security.PublicKey;

import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.impl.util.Base64;

import eu.unicore.samly2.SAMLConstants;
import eu.unicore.samly2.assertion.Assertion;
import eu.unicore.samly2.elements.NameID;
import eu.unicore.samly2.exceptions.SAMLParseException;
import eu.unicore.samly2.proto.AssertionResponse;
import eu.unicore.samly2.proto.AuthnRequest;
import eu.unicore.security.util.client.IAuthenticationConfiguration;

import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.SAMLXMLBeansMapper;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UnsupportedSAMLException;
import xmlbeans.org.oasis.saml2.assertion.AuthnStatementType;
import xmlbeans.org.oasis.saml2.assertion.SubjectType;
import xmlbeans.org.oasis.saml2.protocol.ResponseDocument;

/**
 * Implementation of utility methods for SAML authentication protocol used
 * with SAML HTTP POST binding.
 * @author K. Benedyczak
 */
public class SAMLVOAuthnClientPOST extends AbstractSAMLBase
{
	private String returnURL;
	protected NameID issuer;
	
	/**
	 * Use this constructor if you want to use HTTP POST SAML binding (usual case).
	 * @param address server address to use.  
	 * @param secProv security configuration (used to setup TLS and to sign messages)
	 * @param issuer URI which will be used as SAML request issuer (of SAML type 'entity').
	 * @throws SecuritySetupException
	 * @throws MalformedURLException
	 */
	public SAMLVOAuthnClientPOST(IAuthenticationConfiguration secProv,
			URI issuer, URI returnURL)
		throws SecuritySetupException, MalformedURLException
	{
		super(secProv);
		this.returnURL = returnURL.toString();
		if (issuer == null)
			throw new IllegalArgumentException("No issuer was provided");
		this.issuer = new NameID(issuer.toString(),
				SAMLConstants.NFORMAT_ENTITY);
		doSignRequest = true;
		requireSignedAssertion = true;
	}

	/**
	 * Sets IdP public key for assertion signature verification.
	 * @param publicKey
	 */
	public void setDSigPolicy(PublicKey publicKey)
	{
		this.publicKey = publicKey;
	}

	/**
	 * Don't use this method - it is equivalent to one arg version.
	 */
	@Override
	public void setDSigPolicy(boolean requireSignedResp, 
			boolean requireSignedAssertion, PublicKey publicKey)
	{
		setDSigPolicy(publicKey);
	}

	/**
	 * Extracts subject from authentication assertion and returns 
	 * it wrapped in Identity object.
	 * @param ass authentication assertion.
	 * @return
	 */
	public static Identity getIdentityFromAssertion(Assertion ass)
	{
		AuthnStatementType aS[] = ass.getAuthStatements();
		if (aS == null || aS.length != 1)
			throw new IllegalArgumentException("The assertion passed has zero or" +
					" more then one authentication statements.");
		SubjectType subject = ass.getXML().getAssertion().getSubject();
		if (subject == null)
			throw new IllegalArgumentException("The assertion passed has no subject" +
					"set.");
		if (subject.getNameID() == null)
			throw new IllegalArgumentException("The subjeect in assertion passed has" +
					" no nameID set.");
		return SAMLXMLBeansMapper.map2APIIdentity(subject.getNameID());
	}
	
	/**
	 * Utility to encode to base64 form (ready to be put into 
	 * HTTP form) a SAML authn request.
	 * @param request to be encoded
	 * @return encoded string
	 */
	public static String encodeRequest(AuthnRequest request)
	{
		String assertion = request.getDoc().xmlText();
		return new String(Base64.encode(assertion.getBytes()));
	}

	/**
	 * Utility to decode from base64 string the SAML AssertionResponse.
	 * The response is parsed initially and wrapped as {@link AssertionResponse} 
	 * object.
	 * @param response
	 * @return
	 * @throws SAMLParseException
	 */
	public static AssertionResponse decodeResponse(String response) 
		throws SAMLParseException
	{
		String respString = new String(Base64.decode(response.getBytes()));
		ResponseDocument xmlRespDoc;
		try
		{
			xmlRespDoc = ResponseDocument.Factory.parse(respString);
		} catch (XmlException e)
		{
			throw new SAMLParseException(e.getMessage());
		}
		AssertionResponse resp = new AssertionResponse(xmlRespDoc);
		resp.parse();
		return resp;
	}

	/**
	 * Utility to write default POST form to the writer. The form will be automatically
	 * POSTed to given IdP, with the relayState (optional) and assertion.
	 * @param identityProviderURL where the request will be targeted. In general it will be 
	 * servlet URL.
	 * @param request request to be encoded and sent.
	 * @param relayState arbitrary data (encoded in HTML compatible form!) to be passed
	 * - it should be returned with response without changes. Can be null.
	 */
	public static String writeRequestForm(String identityProviderURL, 
			AuthnRequest request, String relayState)
	{
		String f = formForm.replace("__ACTION__", identityProviderURL);
		f = f.replace("__RELAYSTATE__", relayState == null ? "" : relayState);
		String encodedReq = encodeRequest(request);
		f = f.replace("__SAMLREQUEST__", encodedReq);
		return f;
	}
	
	

	
	/**
	 * Creates a new request. 
	 * @param requestedFormat The format in which identity shall be returned.
	 * See {@link SAMLConstants}.NFORMAT_*.
	 * @return A new signed SAML authentication request. 
	 * @throws InvalidSignatureException
	 */
	public AuthnRequest createRequest(String requestedFormat) 
		throws InvalidSignatureException
	{
		AuthnRequest request = new AuthnRequest(issuer);
		if (requestedFormat != null)
			request.setFormat(requestedFormat);
		request.setConsumerURL(returnURL);

		signIfNeeded(request);
		
		return request;
	}

	
	/**
	 * Does processing of already decoded assertion response and returns 
	 * the first authentication assertion found wrapped into {@link Assertion}.
	 *  
	 * @param resp
	 * @return
	 * @throws SAMLErrorResponseException
	 * @throws SAMLParseException
	 * @throws UnsupportedSAMLException
	 * @throws InvalidSignatureException
	 */
	public Assertion processAuthnResponse(AssertionResponse resp) 
		throws SAMLErrorResponseException, SAMLParseException, 
		UnsupportedSAMLException, InvalidSignatureException
	{
		if (!resp.isStatusOK())
			throw new SAMLErrorResponseException(resp.getErrorMessage(),
					resp.getErrorStatus(), resp.getSubErrorStatus());

		checkResponseSignature(resp);

		Assertion[] assertions;
		try
		{
			assertions = resp.getAssertions();
		} catch (Exception e)
		{
			throw new SAMLParseException(e.toString());
		}
		
		if (assertions.length == 0)
			return null;
		if (assertions.length > 1)
			throw new UnsupportedSAMLException(
				"More than one assertion was returned. It is OK," +
				"however this implementation supports only " +
				"responses with a single assertion.");
		Assertion assertion = assertions[0];

		checkAssertionSignature(resp, assertion);

		return assertion;
	}
	
	/**
	 * Decodes and then processes the response as taken from HTTP POST parameter.
	 * @param response in base64 form.
	 * @return
	 * @throws SAMLParseException
	 * @throws SAMLErrorResponseException
	 * @throws UnsupportedSAMLException
	 * @throws InvalidSignatureException
	 */
	public Assertion decodeAndProcess(String response) throws SAMLParseException,
		SAMLErrorResponseException, UnsupportedSAMLException, InvalidSignatureException
	{
		AssertionResponse resp = decodeResponse(response);
		return processAuthnResponse(resp);
	}
	
	
	private static final String formForm = 
		"<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
		"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\"" +
		"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">" +
		"<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\">" +
		"<body " +
		"onload=\"document.forms[0].submit()\"" +
		">" +
		"<noscript>" +
		"<p>" +
		"<strong>Note:</strong> Since your browser does not support JavaScript," +
		"you must press the Continue button once to proceed." +
		"</p>" +
		"</noscript>" +
		"<form action=\"" +
		"__ACTION__" +
		"\" method=\"post\">" +
		"<div>" +
		"<input type=\"hidden\" name=\"RelayState\" value=\"" +
		"__RELAYSTATE__" +
		"\"/>" +
		"<input type=\"hidden\" name=\"SAMLRequest\" value=\"" +
		"__SAMLREQUEST__" +
		"\"/>" +
		"</div>" +
		"<noscript>" +
		"<div>" +
		"<input type=\"submit\" value=\"Continue\"/>" +
		"</div>" +
		"</noscript>" +
		"</form>" +
		"</body></html>";

}
