/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 24, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.samlapi;

import java.net.MalformedURLException;
import java.net.URI;
import java.security.PublicKey;
import java.util.Properties;

import eu.unicore.samly2.assertion.Assertion;
import eu.unicore.samly2.exceptions.SAMLParseException;
import eu.unicore.samly2.proto.AssertionResponse;
import eu.unicore.security.util.client.IAuthenticationConfiguration;


import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.SAMLAuthnInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UVOSFault;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UnsupportedSAMLException;
import xmlbeans.org.oasis.saml2.protocol.AuthnRequestDocument;
import xmlbeans.org.oasis.saml2.protocol.ResponseDocument;

/**
 * Implementation of utility methods for SAML authentication protocol.
 * See {@link SAMLVOAuthnClientInterface}.
 * 
 * Signatures note: As suggested by spec this implementation:
 * <ul>
 * <li>Always requires signed assertion in response.
 * <li>Always signing the request.
 * <li>Response signature is not checked (not suggested by spec ;-).
 * </ul> 
 * @author K. Benedyczak
 */
public class SAMLVOAuthnClientSOAP extends SAMLVOClient
{
	private SAMLAuthnInterface proxy;
	private SAMLVOAuthnClientPOST util;
	
	/**
	 * Use this constructor if you want to use SAML SOAP binding.
	 * @param address server address to use.  
	 * @param secProv security configuration (used to setup TLS and to sign messages)
	 * @param issuer URI which will be used as SAML request issuer (of SAML type 'entity').
	 * @throws SecuritySetupException
	 * @throws MalformedURLException
	 */
	public SAMLVOAuthnClientSOAP(String address, IAuthenticationConfiguration secProv,
			URI issuer, Properties settings)
		throws SecuritySetupException, MalformedURLException
	{
		super(address, secProv, issuer, settings);
		requireSignedAssertion = true;
		doSignRequest = true;
		requireSignedResp = false;
		try
		{
			if (proxy == null)
				proxy = fact.getSAMLAuthnClient(address);
		} catch (MalformedURLException e){}
		util = new SAMLVOAuthnClientPOST(secProv, issuer, issuer);
	}

	/**
	 * Warning! This method have different semantics as typical setDSigPolicy
	 * used in other SAML clients from this library.
	 * <p>
	 * Requests are always signed, and returned assertions are required to be signed too.
	 * UVOS server will not sign responses.
	 * @param publicKey public key used to verify signature. If null then only signature corectness
	 * will be checked but not if it was created by a trusted party.
	 */
	public void setDSigPolicy(PublicKey publicKey)
	{
		this.publicKey = publicKey;
	}

	/**
	 * Don't use this method - the first two arguments are ignored. Use the
	 * version with only one argument.
	 */
	@Override
	public void setDSigPolicy(boolean a, boolean b, PublicKey publicKey)
	{
		setDSigPolicy(publicKey);
	}

	
	@Override
	public Object getProxy()
	{
		return proxy;
	}

	/**
	 * Performs SOAP request.
	 * @param requestedFormat requested SAML name format to be returned.
	 * @return
	 * @throws SAMLParseException
	 * @throws UnsupportedSAMLException
	 * @throws InvalidSignatureException
	 * @throws InvocationException
	 * @throws SAMLErrorResponseException
	 */
	public Assertion authenticate(String requestedFormat) throws SAMLParseException, 
		UnsupportedSAMLException, InvalidSignatureException, InvocationException,
		SAMLErrorResponseException
	{
		AuthnRequestDocument reqDoc = util.createRequest(requestedFormat).getDoc();
		ResponseDocument xmlRespDoc;
		try
		{
			xmlRespDoc = proxy.authnRequest(reqDoc);
			lastResponse = xmlRespDoc;
		} catch (UVOSFault e)
		{
			throw new InvocationException(e);
		}
		
		AssertionResponse resp = new AssertionResponse(xmlRespDoc);
		resp.parse();
		
		if (!resp.isStatusOK())
			throw new SAMLErrorResponseException(resp.getErrorMessage(),
					resp.getErrorStatus(), resp.getSubErrorStatus());

		checkResponseSignature(resp);

		Assertion[] assertions;
		try
		{
			assertions = resp.getAssertions();
		} catch (Exception e)
		{
			throw new SAMLParseException(e.toString());
		}
		
		if (assertions.length == 0)
			return null;
		if (assertions.length > 1)
			throw new UnsupportedSAMLException(
				"More than one assertion was returned. It is OK," +
				"however this implementation supports only " +
				"responses with a single assertion.");
		Assertion assertion = assertions[0];

		checkAssertionSignature(resp, assertion);

		return assertion;
	}
}
