/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 24, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.samlapi;

import java.net.MalformedURLException;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.apache.xmlbeans.XmlCursor;

import eu.unicore.samly2.SAMLConstants;
import eu.unicore.samly2.elements.NameID;
import eu.unicore.samly2.elements.NameIDPolicy;
import eu.unicore.samly2.exceptions.SAMLParseException;
import eu.unicore.samly2.proto.NameIDMappingRequest;
import eu.unicore.samly2.proto.NameIDMappingResponse;
import eu.unicore.security.util.client.IAuthenticationConfiguration;

import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.SAMLNameIdMappingInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.SAMLXMLBeansMapper;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UnsupportedSAMLException;
import xmlbeans.org.oasis.saml2.protocol.ExtensionsType;
import xmlbeans.org.oasis.saml2.protocol.NameIDMappingResponseDocument;
import xmlbeans.org.oasis.saml2.protocol.NameIDMappingResponseType;
import xmlbeans.pl.edu.icm.samlvo.namemapext.AlternativeNameIDEntryType;
import xmlbeans.pl.edu.icm.samlvo.namemapext.AlternativeNameIDsType;

/**
 * This class should be used by UVOS consumers. It provides high level interface 
 * to SAML identity mapping operations. 
 * <p>
 * For generic usage (e.g. not with UVOS server) there are some limitations:
 * <ul>
 * <li> SAML responses with more than one AssertionStatement are unsupported.
 * </ul>
 * 
 * @author K. Benedyczak
 */
public class SAMLVOMapClient extends SAMLVOClient implements SAMLVOMapClientInterface
{
	private SAMLNameIdMappingInterface mappingProxy;
	
	
	public SAMLVOMapClient(String address, IAuthenticationConfiguration secCfg, Properties settings) 
		throws SecuritySetupException, MalformedURLException
	{
		super(address, secCfg, settings);
		initProxy();
	}

	public SAMLVOMapClient(String address, IAuthenticationConfiguration secCfg, 
			URI issuerURI, Properties settings)
		throws SecuritySetupException, MalformedURLException
	{
		super(address, secCfg, issuerURI, settings);
		initProxy();
	}

	private void initProxy()
	{
		try
		{
			if (mappingProxy == null)
				mappingProxy = fact.getSAMLIdMappingClient(address);
		} catch (MalformedURLException e){}
	}
	
	public Identity mapIdentity(Identity toBeMapped, IdentityType mappedTo) 
		throws InvocationException, InvalidSignatureException, SAMLParseException, 
		SAMLErrorResponseException, UnsupportedSAMLException
	{
		List<Identity> list = mapIdentityGeneric(toBeMapped, 
				mappedTo.getName().toString());
		if (list.size() == 0)
			return null;
		return list.get(0);
	}
	
	public List<Identity> mapIdentity(Identity toBeMapped) 
		throws InvalidSignatureException, InvocationException, 
		SAMLParseException, SAMLErrorResponseException, UnsupportedSAMLException
	{
		return mapIdentityGeneric(toBeMapped, SAMLConstants.NFORMAT_UNSPEC);
	}

	/*-********************************************************
	 * INTERNAL methods 
	 *-********************************************************/
	
	private List<Identity> mapIdentityGeneric(Identity toBeMapped, String format) 
		throws InvalidSignatureException, InvocationException, 
		SAMLParseException, SAMLErrorResponseException, UnsupportedSAMLException
	{
		NameID toMap = new NameID(toBeMapped.getValue(), 
				toBeMapped.getType().getName().toString());
		NameIDPolicy policy = new NameIDPolicy(format);
		NameIDMappingRequest request = new NameIDMappingRequest(issuer,
				toMap, policy);

		signIfNeeded(request);

		NameIDMappingResponseDocument respDoc = null;
		try
		{
			respDoc = mappingProxy.mapNameId(request.getDoc());
			lastResponse = respDoc;
		} catch (Exception e)
		{
			throw new InvocationException(e);
		}

		NameIDMappingResponse resp = new NameIDMappingResponse(respDoc);
		resp.parse();
		
		if (!resp.isStatusOK())
			throw new SAMLErrorResponseException(resp.getErrorMessage(),
					resp.getErrorStatus(), resp.getSubErrorStatus());

		checkResponseSignature(resp);

		NameIDMappingResponseType rr = respDoc.getNameIDMappingResponse();

		Identity baseMap = SAMLXMLBeansMapper.map2APIIdentity(rr.getNameID());
		List<Identity> ret = new ArrayList<Identity>(); 
		ExtensionsType exts = rr.getExtensions();
		if (!addFromExts(ret, exts))
			ret.add(baseMap);
		
		return ret;
	}

	private boolean addFromExts(List<Identity> ret, ExtensionsType exts) 
		throws SAMLParseException
	{
		if (exts == null)
			return false;
		XmlCursor cur = exts.newCursor();
		if (!cur.toFirstChild())
		{
			cur.dispose();
			return false;
		}
		
		do 
		{
			if (cur.getObject() instanceof AlternativeNameIDsType)
			{
				AlternativeNameIDsType allNames = 
					(AlternativeNameIDsType) cur.getObject();
				cur.dispose();
				AlternativeNameIDEntryType entries[] = 
					allNames.getMappedNameIDArray();
				if (entries == null)
					return false;
				for (AlternativeNameIDEntryType entry: entries)
				{
					Identity id = SAMLXMLBeansMapper.map2APIIdentity(
							entry.getNameID());
					ret.add(id);
				}
				return true;
			}
		} while (cur.toNextSibling());
		cur.dispose();
		return false;
	}

	@Override
	public Object getProxy()
	{
		return mappingProxy;
	}
}
