/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 24, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.samlapi;

import java.util.List;

import eu.unicore.samly2.assertion.Assertion;
import eu.unicore.samly2.exceptions.SAMLParseException;

import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.exceptions.AttributeNotFoundException;
import pl.edu.icm.unicore.uvos.api.exceptions.InvalidValueException;
import pl.edu.icm.unicore.uvos.util.AttributeFilter;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UnsupportedSAMLException;

/**
 * @author K. Benedyczak
 */
public interface SAMLVOQueryClientInterface extends SAMLVOClientInterface
{
	/**
	 * Gets all attributes from the list. Attributes in the list must have scopes encoded in
	 * their values. See getSimpleScopedAttribute method documentation for encoding rules.
	 * @param whose owner of attribute
	 * @param attributes which attributes should be get
	 * @return
	 * @throws SAMLParseException
	 * @throws UnsupportedSAMLException
	 * @throws InvalidSignatureException
	 * @throws InvocationException
	 * @throws SAMLErrorResponseException
	 */
	public Assertion getAssertionSimpleScoped(Identity whose, List<Attribute> attributes) 
		throws SAMLParseException, UnsupportedSAMLException, InvalidSignatureException, 
		InvocationException, SAMLErrorResponseException;

	/**
	 * Gets all identity attributes matching to those in the attributes list. If the last
	 * argument is not null then only attributes valid in the specified group are returned. 
	 * Otherwise attributes are returned regardless of their scope. 
	 * @param whose owner of attribute
	 * @param attributes which attributes should be get
	 * @param scope optinal scope to which query should be limited
	 * @return
	 * @throws SAMLParseException
	 * @throws UnsupportedSAMLException
	 * @throws InvalidSignatureException
	 * @throws InvocationException
	 * @throws SAMLErrorResponseException
	 */
	public Assertion getAssertion(Identity whose, List<Attribute> attributes, Group scope) 
		throws SAMLParseException, UnsupportedSAMLException, InvalidSignatureException, 
		InvocationException, SAMLErrorResponseException;

	
	/**
	 * Gets attribute assertion with all attributes with the given name 
	 * regardless of their scope.  
	 * @param whose owner of attribute
	 * @param attributeName name of requested attribute
	 * @return assertion
	 * @throws SAMLErrorResponseException 
	 * @throws InvocationException 
	 * @throws InvalidSignatureException 
	 * @throws UnsupportedSAMLException 
	 * @throws SAMLParseException 
	 */
	public abstract Assertion getAssertion(Identity whose,
			String attributeName) throws SAMLParseException, UnsupportedSAMLException,
			InvalidSignatureException, InvocationException, SAMLErrorResponseException;

	/**
	 * Gets attribute assertion with all attributes.
	 * @param whose owner of attribute
	 * @return assertion
	 * @throws Exception 
	 */
	public abstract Assertion getAssertion(Identity whose) throws Exception;

	/**
	 * Gets attribute assertion with all attributes valid in 
	 * scope of the specified group.
	 * @param whose owner of attribute
	 * @param scope group scope
	 * @return
	 * @throws SAMLErrorResponseException 
	 * @throws InvocationException 
	 * @throws InvalidSignatureException 
	 * @throws UnsupportedSAMLException 
	 * @throws SAMLParseException 
	 */
	public abstract Assertion getAssertion(Identity whose,
		Group scope) throws SAMLParseException, UnsupportedSAMLException, 
		InvalidSignatureException, InvocationException, SAMLErrorResponseException;
	
	/**
	 * Gets attribute assertion with all attributes with the given name 
	 * regardless of their scope.
	 * This version allows for specifying required attribute values too.
	 * @param whose owner of attribute
	 * @param attributeName name of requested attribute
	 * @return assertion
	 * @throws SAMLErrorResponseException 
	 * @throws InvocationException 
	 * @throws InvalidSignatureException 
	 * @throws UnsupportedSAMLException 
	 * @throws SAMLParseException 
	 */
	public abstract Assertion getAssertion(Identity whose,
			Attribute attribute) throws SAMLParseException, 
			UnsupportedSAMLException, InvalidSignatureException, InvocationException, SAMLErrorResponseException;

	
	/**
	 * Gets all attributes with the given name regardless of their scope.  
	 * @param whose owner of attribute
	 * @param attributeName name of requested attribute
	 * @return attribute
	 * @throws SAMLErrorResponseException 
	 * @throws InvocationException 
	 * @throws InvalidSignatureException 
	 * @throws UnsupportedSAMLException 
	 * @throws SAMLParseException 
	 */
	public abstract List<Attribute> getAttributes(Identity whose,
			String attributeName) throws SAMLParseException, UnsupportedSAMLException, InvalidSignatureException, InvocationException, SAMLErrorResponseException;

	/**
	 * Gets attribute with the given name valid in scope of the specified group.
	 * @param whose owner of attribute
	 * @param attributeName name of requested attribute
	 * @param scope group scope
	 * @return
	 * @throws SAMLErrorResponseException 
	 * @throws InvocationException 
	 * @throws InvalidSignatureException 
	 * @throws UnsupportedSAMLException 
	 * @throws SAMLParseException 
	 */
	public abstract Attribute getAttribute(Identity whose, Group scope,
			String attributeName) throws SAMLParseException, UnsupportedSAMLException, InvalidSignatureException, InvocationException, SAMLErrorResponseException;

	/**
	 * Gets all attributes with the given name regardless of their scope.
	 * This version allows for specifing required attribute values too.
	 * @param whose owner of attribute
	 * @param attributeName name of requested attribute
	 * @return attribute
	 * @throws SAMLErrorResponseException 
	 * @throws InvocationException 
	 * @throws InvalidSignatureException 
	 * @throws UnsupportedSAMLException 
	 * @throws SAMLParseException 
	 */
	public abstract List<Attribute> getAttributes(Identity whose,
			Attribute attribute) throws SAMLParseException, UnsupportedSAMLException, InvalidSignatureException, InvocationException, SAMLErrorResponseException;

	/**
	 * Gets attribute with the given name valid in scope of the specified group.
	 * This version allows for specifing required attribute values too.
	 * @param whose owner of attribute
	 * @param attributeName name of requested attribute
	 * @param scope group scope
	 * @return
	 * @throws SAMLErrorResponseException 
	 * @throws InvocationException 
	 * @throws InvalidSignatureException 
	 * @throws UnsupportedSAMLException 
	 * @throws SAMLParseException 
	 */
	public abstract Attribute getAttribute(Identity whose, Group scope,
			Attribute attribute) throws SAMLParseException, UnsupportedSAMLException, InvalidSignatureException, InvocationException, SAMLErrorResponseException;

	/**
	 * Gets attributes with the given name valid in scope of the specified group(s).
	 * <p>
	 * Using this version the scope must be already encoded in attribute values (therefore
	 * there must be at least one value defined) in the following way:
	 * <pre> [actualValue]@/[group[/subgroup[...]]] </pre>
	 * In case of query for global attributes finish value with '@/' alone.
	 * <p>
	 * Note that it is possible to query for values which are valid in different scopes;
	 * then all those values will be put as different elements of the returned 
	 * attributes list.
	 * <p>
	 * It is possible to use this method to ask for attribute without a value
	 * using the syntax:
	 * <pre> @/[group[/subgroup[...]]] </pre>
	 * However it is impossible to query for all values of the given attribute name 
	 * (other methods of this interface allow for doing this).
	 *  
	 * @param whose owner of attribute
	 * @param attributeName name of requested attribute
	 * @param scope group scope
	 * @return
	 * @throws SAMLErrorResponseException 
	 * @throws InvocationException 
	 * @throws InvalidSignatureException 
	 * @throws UnsupportedSAMLException 
	 * @throws SAMLParseException
	 * @throws {@link InvalidValueException} when encoding of 
	 * 	values is invalid or there is none of them.
	 */
	public abstract List<Attribute> getSimpleScopedAttribute(Identity whose, Attribute attribute)
		throws SAMLParseException, UnsupportedSAMLException, InvalidSignatureException, 
		InvocationException, SAMLErrorResponseException, InvalidValueException;

	
	/**
	 * Gets all attributes regardless of their scope.
	 * @param whose owner of attribute
	 * @return
	 * @throws Exception 
	 */
	public abstract List<Attribute> getAttributes(Identity whose) throws Exception;

	/**
	 * Gets all attributes valid in scope of the specified group.
	 * @param whose owner of attribute
	 * @param scope group scope
	 * @return
	 * @throws SAMLErrorResponseException 
	 * @throws InvocationException 
	 * @throws InvalidSignatureException 
	 * @throws UnsupportedSAMLException 
	 * @throws SAMLParseException 
	 */
	public abstract List<Attribute> getAttributes(Identity whose,
			Group scope) throws SAMLParseException, UnsupportedSAMLException, InvalidSignatureException, InvocationException, SAMLErrorResponseException;
	
	/**
	 * This method retrieves a list of attributes from the SAML endpoint in an 
	 * interoperable way. First an unconstrained query is made. If neither filter
	 * nor scope is provided then this is returned. Otherwise attributes obtained 
	 * from the initial query are filtered (with group and name/value filter) and
	 * another query is constructed to get only those remaining attributes. 
	 * <p>
	 * This method guarantee that all attributes used in the second query are encoded
	 * in the same way in which the server returned them. Therefore interoperability
	 * with non-UVOS servers is ensured. However note that to be able to filter
	 * attributes by scope, the attribute must be in one of supported profiles, as
	 * scopes are not a generic SAML feature.    
	 * @param whose subject
	 * @param scope if not null, then only attributes from this group are returned.
	 * @param matchSubgroups relevant only if scope != null, then if true also 
	 * attributes valid in subgroups of the 'scope' group are returned.
	 * @param filter name/value filter, may be null if no filtering is needed.
	 * @return
	 * @throws SAMLErrorResponseException 
	 * @throws InvocationException 
	 * @throws InvalidSignatureException 
	 * @throws UnsupportedSAMLException 
	 * @throws SAMLParseException 
	 * @throws AttributeNotFoundException when the filter removed all available attributes.
	 */
	public Assertion getFilteredAssertion(Identity whose, Group scope, 
			boolean matchSubgroups, AttributeFilter filter) 
			throws SAMLParseException, UnsupportedSAMLException, 
			InvalidSignatureException, InvocationException, 
			SAMLErrorResponseException, AttributeNotFoundException;
}