/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Apr 12, 2008
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.uvos.test;

import junit.framework.TestCase;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.api.Permissions;
import pl.edu.icm.unicore.uvos.db.InitDB;
import pl.edu.icm.unicore.uvos.engine.VirtualSecurityContextImpl;
import pl.edu.icm.unicore.uvos.engine.api.UVOSManagementInterface;
import pl.edu.icm.unicore.uvos.server.ComponentContainer;
import pl.edu.icm.unicore.uvos.server.VOServer;
import pl.edu.icm.unicore.uvos.server.idresolver.Authenticator;
import pl.edu.icm.unicore.uvos.util.Configuration;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSClientFactory;
import pl.edu.icm.unicore.uvos.wsclient.api.WSManagementClient;

/**
 * AuthN tests with server configuration with failOnError=true and normal SSL authN
 * @author K. Benedyczak
 */
public class AuthN2Test extends TestCase
{
	private static final String UVOS_CONF = "src/test/resources/uvos/uvosServerAuthn1.conf";

	private static boolean initDone = false;
	protected String addr;
	protected TestSecurityConfig conf, confWrongCert;
	private ComponentContainer container;
	
	public AuthN2Test()
	{
		container = new ComponentContainer(UVOS_CONF);
	}
	
	@Override
	protected void setUp()
	{
		try
		{
			System.setProperty("crlmanager.properties.file", "src/test/resources/certs/crlcheck.properties");
			Configuration c = new Configuration("src/test/resources/functionalTest.conf");
			conf = new TestSecurityConfig(c);
			Configuration c2 = new Configuration("src/test/resources/authnTest1.conf");
			confWrongCert = new TestSecurityConfig(c2);
	
			addr = conf.getProtocol() + "://" + 
				conf.getServerHost() + ":" + conf.getServerPort();
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail();
		}

		if (initDone)
			return;
		try
		{
			InitDB.main(new String[]{UVOS_CONF, "-b"});
			VOServer serverInstance = container.getComponent(VOServer.class);
			serverInstance.startup(container.getContainer());
			
			UVOSManagementInterface man = container.getComponent(UVOSManagementInterface.class); 
			Identity testing = new Identity(IdentityType.DN, 
				"cn=example client,ou=example ou,o=example o,l=example city,st=example province,c=pl");
			Authenticator.injectFakeContext(new VirtualSecurityContextImpl(Permissions.ALL_P));
			man.addIdentity(testing);
			testing = new Identity(IdentityType.DN, 
				"cn=test user,ou=unknown,o=icm,l=unknown,st=unknown,c=pl");
			man.addIdentity(testing);
			
			testing = new Identity(IdentityType.DN, 
				"CN=TestRevocation, OU=PL-Grid Internal, O=ICM, C=PL");
			man.addIdentity(testing);
			Authenticator.injectFakeContext(null);
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail();
		}
		initDone = true;
	}
	
	public void testExpiredCert()
	{
		try
		{
			System.out.println("\nChecking access with expired cert\n");
			confWrongCert.doSSLAuthn = true;
			UVOSClientFactory fact = new UVOSClientFactory(confWrongCert, 
					confWrongCert.getConfiguration());
			WSManagementClient voMan = fact.getWSManagementClientHL(addr);
			voMan.getAttributeTypes();
			System.out.println("!!!! Got access with SSL authN with expired cert.");
			fail("Got access with SSL authN with expired cert.");
		} catch (Exception e){}
	}
	
	public void testRevokedCert()
	{
		try
		{
			System.out.println("\nChecking access with revoked cert\n");
			Configuration c = new Configuration("src/test/resources/authnTest2.conf");
			TestSecurityConfig confRevokedCert = new TestSecurityConfig(c);
			confRevokedCert.doSSLAuthn = true;
			UVOSClientFactory fact = new UVOSClientFactory(confRevokedCert, 
					confRevokedCert.getConfiguration());
			WSManagementClient voMan = fact.getWSManagementClientHL(addr);
			voMan.getAttributeTypes();
			System.out.println("!!!! Got access with SSL authN with expired cert.");
			fail("Got access with SSL authN with expired cert.");
		} catch (Exception e){}
	}
	
	
	public void testSSLLacking()
	{
		try
		{
			System.out.println("\nChecking access without SSL client " +
					"authN while it is required by server.\n");
			conf.doSSLAuthn = false;
			UVOSClientFactory fact = new UVOSClientFactory(conf, conf.getConfiguration());
			WSManagementClient voMan = fact.getWSManagementClientHL(addr);
			voMan.getAttributeTypes();
			System.out.println("!!!!!Got access without SSL authN while was required by server.");
			fail("Got access without SSL authN while was required by server.");
		} catch (Exception e){}
	}
	
	public void testChainFailures2()
	{
		try
		{
			System.out.println("\nChecking authN failures in chain 2\n");
			conf.doSSLAuthn = true;
			conf.doHttpAuthn = true;
			conf.httpPasswd = "wrong password";
			conf.httpUser = "admin@localhost";
			UVOSClientFactory fact = new UVOSClientFactory(conf, conf.getConfiguration());
			WSManagementClient voMan = fact.getWSManagementClientHL(addr);
			voMan.getAttributeTypes();
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("Didn't get access with correct SSL authN and " +
				"incorrect HTTP while the later should be ignored.");		
		}
	}
	
	public void testSSLOK()
	{
		try
		{
			System.out.println("\nChecking access with correct SSL client " +
				"authN.\n");
			conf.doSSLAuthn = true;
			UVOSClientFactory fact = new UVOSClientFactory(conf, conf.getConfiguration());
			WSManagementClient voMan = fact.getWSManagementClientHL(addr);
			voMan.getAttributeTypes();
			
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("Didn't get access with SSL authN");
		}
		
	}
}
